package fr.sanchezm.attestationsCovid19.utilities

import android.content.res.AssetManager
import android.util.Log
import com.tom_roush.pdfbox.pdmodel.PDDocument
import com.tom_roush.pdfbox.pdmodel.PDPage
import com.tom_roush.pdfbox.pdmodel.PDPageContentStream
import com.tom_roush.pdfbox.pdmodel.common.PDRectangle
import com.tom_roush.pdfbox.pdmodel.font.PDType1Font
import com.tom_roush.pdfbox.pdmodel.graphics.image.LosslessFactory
import com.tom_roush.pdfbox.pdmodel.interactive.form.PDAcroForm
import com.tom_roush.pdfbox.pdmodel.interactive.form.PDCheckbox
import com.tom_roush.pdfbox.pdmodel.interactive.form.PDTextField
import fr.sanchezm.attestationsCovid19.data.db.entity.Attestation
import java.io.File
import java.io.IOException


class PdfUtils private constructor(
    private val assetManager: AssetManager,
    private val path: String
) {

    // private val checkboxListNames = ArrayList<String>()
    // private val font = PDType1Font.HELVETICA

    fun fillForm(attestation: Attestation, callback: (Boolean) -> Unit) {
        checkFolder()
        try {
            // Load the document and get the AcroForm
            val document: PDDocument = PDDocument.load(assetManager.open("attestation.pdf"))
            val docCatalog = document.documentCatalog
            val acroForm = docCatalog.acroForm

            // Adding data
            if (setFieldsData(acroForm, attestation)) {
                setCheckboxFields(acroForm, attestation)
                addPageWithQrCode(document, attestation)

                Log.v("PdfUtils", "Save File to ${getPath(attestation.createAt)}")
                document.save(getPath(attestation.createAt))
                document.close()
                callback(true)
            } else {
                callback(false)
            }
        } catch (e: IOException) {
            e.printStackTrace()
            callback(false)
        }
    }

    fun getPath(createAt: Long): String = getPath(path, createAt)

    private fun addPageWithQrCode(document: PDDocument, attestation: Attestation) {
        val size = 360
        val margin = 25f

//      Add QRCode on the first page
        val page1: PDPage = document.pages.first()
        Log.v("PdfUtils", "Width: ${page1.mediaBox.width}; Height: ${page1.mediaBox.height}")
        addQrCode(
            document,
            page1,
            attestation,
            size / 3,
            page1.mediaBox.upperRightX - margin - size / 3,
            25f
        )

        // Add QRCode on a new page
        val page2 = PDPage(PDRectangle.A4)

        document.addPage(page2)
        addQrCode(document, page2, attestation, size, margin, page2.mediaBox.height - (size + margin))

    }

    private fun addQrCode(
        document: PDDocument,
        page: PDPage,
        attestation: Attestation,
        size: Int,
        x: Float,
        y: Float
    ) {

        val contentStream = PDPageContentStream(document, page, true, false)
        val alphaXimage =
            LosslessFactory.createFromImage(document, QRCodeUtils.getQrCode(attestation, size))

        contentStream.drawImage(alphaXimage, x, y)
        contentStream.close()
    }

    private fun setFieldsData(acroForm: PDAcroForm, attestation: Attestation): Boolean {
        val profile = attestation.profile

        try {
            // Init Fields
            val namesField = acroForm.getField("Nom et prénom") as PDTextField
            val birthdayField = acroForm.getField("Date de naissance") as PDTextField
            val birthPlaceField = acroForm.getField("Lieu de naissance") as PDTextField
            val addressField = acroForm.getField("Adresse") as PDTextField
            val cityField = acroForm.getField("Ville") as PDTextField
            val dateField = acroForm.getField("Date") as PDTextField
            val exitField = acroForm.getField("Heure") as PDTextField

            namesField.value = "${profile.firstName} ${profile.lastName}"
            birthdayField.value = profile.birthday
            birthPlaceField.value = profile.birthPlace
            addressField.value = "${profile.address}, ${profile.city} ${profile.postalCode}"
            cityField.value = profile.city
            dateField.value = attestation.exitDate
            exitField.value = attestation.exitHour.replace('h', ':')
            return true
        } catch (e: Exception) {
            Log.e(TAG, "${e.message}")
        }
        return false
    }

    private fun setCheckboxFields(acroForm: PDAcroForm, attestation: Attestation) {
        attestation.reasons.forEach {
            (acroForm.getField("distinction Motif $it") as PDCheckbox).check()
        }
    }

    private fun checkFolder() {
        val file = "$path/$folderPath"

        if (!File(file).exists()) {
            File(file).mkdir()
        }
    }

    companion object {
        @Volatile
        private var instance: PdfUtils? = null
        private val LOCK = Any()
        private const val folderPath = "attestations"
        private val TAG = PdfUtils::class.simpleName

        fun getInstance() = instance

        fun getInstance(assetManager: AssetManager, path: String) = instance ?: synchronized(LOCK) {
            instance ?: PdfUtils(assetManager, path).also { instance = it }
        }

        fun getPath(path: String, createAt: Long): String = "$path/$folderPath/$createAt.pdf"

    }
}